<?php

/**
 * @file
 * Contains \Drupal\masquerade_nominate\Plugin\Block\MasqueradeNominateBlock
 * actually might be able to use the normal masquerade block...
 * .
 */

namespace Drupal\masquerade_nominate\Plugin\Block;

use Drupal\Core\Access\AccessResult;
use Drupal\masquerade\Plugin\Block\MasqueradeBlock;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Url;
use Drupal\Core\Session\AccountInterface;
use Drupal\user\Entity\User;

/**
 * Allows ordinary users to masquerade when nominated by other users.
 *
 * @Block(
 *   id = "masquerade_nominate",
 *   admin_label = @Translation("Masquerade as nominand"),
 *   category = @Translation("Forms")
 * )
 */
class MasqueradeNominateBlock extends MasqueradeBlock implements ContainerFactoryPluginInterface {

  private $targets;
  private $currentUser;

  /**
   * Constructs a new MasqueradeBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   * @param \Drupal\masquerade\Masquerade $masquerade
   *   The masquerade service.
   * @param \Drupal\Core\Session\Account $account
   *   The currentUser service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, $form_builder, $masquerade, AccountInterface $account) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $form_builder, $masquerade);
    $this->currentUser = $account;
    $this->targets = masquerade_nominate_masquerade_as($account);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('form_builder'),
      $container->get('masquerade'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    $result = parent::blockAccess($account);
    if ($result->isForbidden() and !$this->masquerade->isMasquerading()) {
      $result = Accessresult::allowedIf($this->currentUser->hasPermission('masquerade as any user') or (count($this->targets)))
        ->addCacheContexts(['session.is_masquerading']);
    }
    return $result;

  }


  /**
   * {@inheritdoc}
   */
  public function build() {
    // Normal masquerade block for permitted users.
    if ($this->currentUser->hasPermission('masquerade as any user')) {
      return parent::build();
    }
    // If the user has any permissions to masquerade use the default block
    if (AccessResult::allowedIfHasPermissions($account, $this->masquerade->getPermissions(), 'OR')->isAllowed()) {
      return parent::build();
    }

echo "targets";print_r($this->targets);
    $links = [];
    foreach (User::loadMultiple($this->targets) as $account) {
      $links['#items'] = \Drupal::l(
        $account->getUserName(),
        Url::fromRoute(
          'entity.user.masquerade',
          ['user' => $account->id()]
          //['query' => \Drupal::destination()->getAsArray()]//this makes it worth hardly caching the block
        )
      );
    }
    if (count($links)) {
      return [
        '#theme' => 'item_list',
        '#items' => $links
      ];
    }
  }
}
